# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['ReplicationVaultArgs', 'ReplicationVault']

@pulumi.input_type
class ReplicationVaultArgs:
    def __init__(__self__, *,
                 replication_source_region_id: pulumi.Input[str],
                 replication_source_vault_id: pulumi.Input[str],
                 vault_name: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 vault_storage_class: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ReplicationVault resource.
        :param pulumi.Input[str] replication_source_region_id: The ID of the region where the source vault resides.
        :param pulumi.Input[str] replication_source_vault_id: The ID of the source vault.
        :param pulumi.Input[str] vault_name: The name of the backup vault. The name must be 1 to 64 characters in length.
        :param pulumi.Input[str] description: The description of the backup vault. The description must be 0 to 255 characters in length.
        :param pulumi.Input[str] vault_storage_class: The storage type of the backup vault. Valid values: `STANDARD`.
        """
        pulumi.set(__self__, "replication_source_region_id", replication_source_region_id)
        pulumi.set(__self__, "replication_source_vault_id", replication_source_vault_id)
        pulumi.set(__self__, "vault_name", vault_name)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if vault_storage_class is not None:
            pulumi.set(__self__, "vault_storage_class", vault_storage_class)

    @property
    @pulumi.getter(name="replicationSourceRegionId")
    def replication_source_region_id(self) -> pulumi.Input[str]:
        """
        The ID of the region where the source vault resides.
        """
        return pulumi.get(self, "replication_source_region_id")

    @replication_source_region_id.setter
    def replication_source_region_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "replication_source_region_id", value)

    @property
    @pulumi.getter(name="replicationSourceVaultId")
    def replication_source_vault_id(self) -> pulumi.Input[str]:
        """
        The ID of the source vault.
        """
        return pulumi.get(self, "replication_source_vault_id")

    @replication_source_vault_id.setter
    def replication_source_vault_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "replication_source_vault_id", value)

    @property
    @pulumi.getter(name="vaultName")
    def vault_name(self) -> pulumi.Input[str]:
        """
        The name of the backup vault. The name must be 1 to 64 characters in length.
        """
        return pulumi.get(self, "vault_name")

    @vault_name.setter
    def vault_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "vault_name", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the backup vault. The description must be 0 to 255 characters in length.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="vaultStorageClass")
    def vault_storage_class(self) -> Optional[pulumi.Input[str]]:
        """
        The storage type of the backup vault. Valid values: `STANDARD`.
        """
        return pulumi.get(self, "vault_storage_class")

    @vault_storage_class.setter
    def vault_storage_class(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vault_storage_class", value)


@pulumi.input_type
class _ReplicationVaultState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 replication_source_region_id: Optional[pulumi.Input[str]] = None,
                 replication_source_vault_id: Optional[pulumi.Input[str]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 vault_name: Optional[pulumi.Input[str]] = None,
                 vault_storage_class: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ReplicationVault resources.
        :param pulumi.Input[str] description: The description of the backup vault. The description must be 0 to 255 characters in length.
        :param pulumi.Input[str] replication_source_region_id: The ID of the region where the source vault resides.
        :param pulumi.Input[str] replication_source_vault_id: The ID of the source vault.
        :param pulumi.Input[str] status: The status of the resource.
        :param pulumi.Input[str] vault_name: The name of the backup vault. The name must be 1 to 64 characters in length.
        :param pulumi.Input[str] vault_storage_class: The storage type of the backup vault. Valid values: `STANDARD`.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if replication_source_region_id is not None:
            pulumi.set(__self__, "replication_source_region_id", replication_source_region_id)
        if replication_source_vault_id is not None:
            pulumi.set(__self__, "replication_source_vault_id", replication_source_vault_id)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if vault_name is not None:
            pulumi.set(__self__, "vault_name", vault_name)
        if vault_storage_class is not None:
            pulumi.set(__self__, "vault_storage_class", vault_storage_class)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the backup vault. The description must be 0 to 255 characters in length.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="replicationSourceRegionId")
    def replication_source_region_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the region where the source vault resides.
        """
        return pulumi.get(self, "replication_source_region_id")

    @replication_source_region_id.setter
    def replication_source_region_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "replication_source_region_id", value)

    @property
    @pulumi.getter(name="replicationSourceVaultId")
    def replication_source_vault_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the source vault.
        """
        return pulumi.get(self, "replication_source_vault_id")

    @replication_source_vault_id.setter
    def replication_source_vault_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "replication_source_vault_id", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The status of the resource.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter(name="vaultName")
    def vault_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the backup vault. The name must be 1 to 64 characters in length.
        """
        return pulumi.get(self, "vault_name")

    @vault_name.setter
    def vault_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vault_name", value)

    @property
    @pulumi.getter(name="vaultStorageClass")
    def vault_storage_class(self) -> Optional[pulumi.Input[str]]:
        """
        The storage type of the backup vault. Valid values: `STANDARD`.
        """
        return pulumi.get(self, "vault_storage_class")

    @vault_storage_class.setter
    def vault_storage_class(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vault_storage_class", value)


class ReplicationVault(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 replication_source_region_id: Optional[pulumi.Input[str]] = None,
                 replication_source_vault_id: Optional[pulumi.Input[str]] = None,
                 vault_name: Optional[pulumi.Input[str]] = None,
                 vault_storage_class: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a Hybrid Backup Recovery (HBR) Replication Vault resource.

        For information about Hybrid Backup Recovery (HBR) Replication Vault and how to use it, see [What is Replication Vault](https://www.alibabacloud.com/help/en/doc-detail/345603.html).

        > **NOTE:** Available in v1.152.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        source_region = config.get("sourceRegion")
        if source_region is None:
            source_region = "cn-hangzhou"
        source = alicloud.Provider("source", region=source_region)
        default_replication_vault_regions = alicloud.hbr.get_replication_vault_regions()
        replication = alicloud.Provider("replication", region=default_replication_vault_regions.regions[0].replication_region_id)
        default_vault = alicloud.hbr.Vault("defaultVault", vault_name="terraform-example",
        opts=pulumi.ResourceOptions(provider=alicloud["source"]))
        default_replication_vault = alicloud.hbr.ReplicationVault("defaultReplicationVault",
            replication_source_region_id=source_region,
            replication_source_vault_id=default_vault.id,
            vault_name="terraform-example",
            vault_storage_class="STANDARD",
            description="terraform-example",
            opts=pulumi.ResourceOptions(provider=alicloud["replication"]))
        ```

        ## Import

        Hybrid Backup Recovery (HBR) Replication Vault can be imported using the id, e.g.

        ```sh
         $ pulumi import alicloud:hbr/replicationVault:ReplicationVault example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the backup vault. The description must be 0 to 255 characters in length.
        :param pulumi.Input[str] replication_source_region_id: The ID of the region where the source vault resides.
        :param pulumi.Input[str] replication_source_vault_id: The ID of the source vault.
        :param pulumi.Input[str] vault_name: The name of the backup vault. The name must be 1 to 64 characters in length.
        :param pulumi.Input[str] vault_storage_class: The storage type of the backup vault. Valid values: `STANDARD`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ReplicationVaultArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Hybrid Backup Recovery (HBR) Replication Vault resource.

        For information about Hybrid Backup Recovery (HBR) Replication Vault and how to use it, see [What is Replication Vault](https://www.alibabacloud.com/help/en/doc-detail/345603.html).

        > **NOTE:** Available in v1.152.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        source_region = config.get("sourceRegion")
        if source_region is None:
            source_region = "cn-hangzhou"
        source = alicloud.Provider("source", region=source_region)
        default_replication_vault_regions = alicloud.hbr.get_replication_vault_regions()
        replication = alicloud.Provider("replication", region=default_replication_vault_regions.regions[0].replication_region_id)
        default_vault = alicloud.hbr.Vault("defaultVault", vault_name="terraform-example",
        opts=pulumi.ResourceOptions(provider=alicloud["source"]))
        default_replication_vault = alicloud.hbr.ReplicationVault("defaultReplicationVault",
            replication_source_region_id=source_region,
            replication_source_vault_id=default_vault.id,
            vault_name="terraform-example",
            vault_storage_class="STANDARD",
            description="terraform-example",
            opts=pulumi.ResourceOptions(provider=alicloud["replication"]))
        ```

        ## Import

        Hybrid Backup Recovery (HBR) Replication Vault can be imported using the id, e.g.

        ```sh
         $ pulumi import alicloud:hbr/replicationVault:ReplicationVault example <id>
        ```

        :param str resource_name: The name of the resource.
        :param ReplicationVaultArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ReplicationVaultArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 replication_source_region_id: Optional[pulumi.Input[str]] = None,
                 replication_source_vault_id: Optional[pulumi.Input[str]] = None,
                 vault_name: Optional[pulumi.Input[str]] = None,
                 vault_storage_class: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ReplicationVaultArgs.__new__(ReplicationVaultArgs)

            __props__.__dict__["description"] = description
            if replication_source_region_id is None and not opts.urn:
                raise TypeError("Missing required property 'replication_source_region_id'")
            __props__.__dict__["replication_source_region_id"] = replication_source_region_id
            if replication_source_vault_id is None and not opts.urn:
                raise TypeError("Missing required property 'replication_source_vault_id'")
            __props__.__dict__["replication_source_vault_id"] = replication_source_vault_id
            if vault_name is None and not opts.urn:
                raise TypeError("Missing required property 'vault_name'")
            __props__.__dict__["vault_name"] = vault_name
            __props__.__dict__["vault_storage_class"] = vault_storage_class
            __props__.__dict__["status"] = None
        super(ReplicationVault, __self__).__init__(
            'alicloud:hbr/replicationVault:ReplicationVault',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            replication_source_region_id: Optional[pulumi.Input[str]] = None,
            replication_source_vault_id: Optional[pulumi.Input[str]] = None,
            status: Optional[pulumi.Input[str]] = None,
            vault_name: Optional[pulumi.Input[str]] = None,
            vault_storage_class: Optional[pulumi.Input[str]] = None) -> 'ReplicationVault':
        """
        Get an existing ReplicationVault resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the backup vault. The description must be 0 to 255 characters in length.
        :param pulumi.Input[str] replication_source_region_id: The ID of the region where the source vault resides.
        :param pulumi.Input[str] replication_source_vault_id: The ID of the source vault.
        :param pulumi.Input[str] status: The status of the resource.
        :param pulumi.Input[str] vault_name: The name of the backup vault. The name must be 1 to 64 characters in length.
        :param pulumi.Input[str] vault_storage_class: The storage type of the backup vault. Valid values: `STANDARD`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ReplicationVaultState.__new__(_ReplicationVaultState)

        __props__.__dict__["description"] = description
        __props__.__dict__["replication_source_region_id"] = replication_source_region_id
        __props__.__dict__["replication_source_vault_id"] = replication_source_vault_id
        __props__.__dict__["status"] = status
        __props__.__dict__["vault_name"] = vault_name
        __props__.__dict__["vault_storage_class"] = vault_storage_class
        return ReplicationVault(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        The description of the backup vault. The description must be 0 to 255 characters in length.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="replicationSourceRegionId")
    def replication_source_region_id(self) -> pulumi.Output[str]:
        """
        The ID of the region where the source vault resides.
        """
        return pulumi.get(self, "replication_source_region_id")

    @property
    @pulumi.getter(name="replicationSourceVaultId")
    def replication_source_vault_id(self) -> pulumi.Output[str]:
        """
        The ID of the source vault.
        """
        return pulumi.get(self, "replication_source_vault_id")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The status of the resource.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="vaultName")
    def vault_name(self) -> pulumi.Output[str]:
        """
        The name of the backup vault. The name must be 1 to 64 characters in length.
        """
        return pulumi.get(self, "vault_name")

    @property
    @pulumi.getter(name="vaultStorageClass")
    def vault_storage_class(self) -> pulumi.Output[str]:
        """
        The storage type of the backup vault. Valid values: `STANDARD`.
        """
        return pulumi.get(self, "vault_storage_class")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['PluginArgs', 'Plugin']

@pulumi.input_type
class PluginArgs:
    def __init__(__self__, *,
                 plugin_data: pulumi.Input[str],
                 plugin_name: pulumi.Input[str],
                 plugin_type: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, Any]]] = None):
        """
        The set of arguments for constructing a Plugin resource.
        :param pulumi.Input[str] plugin_data: The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        :param pulumi.Input[str] plugin_name: The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        :param pulumi.Input[str] plugin_type: The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
               - ipControl: indicates IP address-based access control.
               - trafficControl: indicates throttling.
               - backendSignature: indicates backend signature.
               - jwtAuth: indicates JWT (OpenId Connect).
               - cors: indicates cross-origin resource access (CORS).
               - caching: indicates caching.
        :param pulumi.Input[str] description: The description of the plug-in, which cannot exceed 200 characters.
        :param pulumi.Input[Mapping[str, Any]] tags: A mapping of tags to assign to the resource.
        """
        pulumi.set(__self__, "plugin_data", plugin_data)
        pulumi.set(__self__, "plugin_name", plugin_name)
        pulumi.set(__self__, "plugin_type", plugin_type)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter(name="pluginData")
    def plugin_data(self) -> pulumi.Input[str]:
        """
        The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        """
        return pulumi.get(self, "plugin_data")

    @plugin_data.setter
    def plugin_data(self, value: pulumi.Input[str]):
        pulumi.set(self, "plugin_data", value)

    @property
    @pulumi.getter(name="pluginName")
    def plugin_name(self) -> pulumi.Input[str]:
        """
        The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        """
        return pulumi.get(self, "plugin_name")

    @plugin_name.setter
    def plugin_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "plugin_name", value)

    @property
    @pulumi.getter(name="pluginType")
    def plugin_type(self) -> pulumi.Input[str]:
        """
        The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
        - ipControl: indicates IP address-based access control.
        - trafficControl: indicates throttling.
        - backendSignature: indicates backend signature.
        - jwtAuth: indicates JWT (OpenId Connect).
        - cors: indicates cross-origin resource access (CORS).
        - caching: indicates caching.
        """
        return pulumi.get(self, "plugin_type")

    @plugin_type.setter
    def plugin_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "plugin_type", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the plug-in, which cannot exceed 200 characters.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _PluginState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 plugin_data: Optional[pulumi.Input[str]] = None,
                 plugin_name: Optional[pulumi.Input[str]] = None,
                 plugin_type: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, Any]]] = None):
        """
        Input properties used for looking up and filtering Plugin resources.
        :param pulumi.Input[str] description: The description of the plug-in, which cannot exceed 200 characters.
        :param pulumi.Input[str] plugin_data: The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        :param pulumi.Input[str] plugin_name: The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        :param pulumi.Input[str] plugin_type: The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
               - ipControl: indicates IP address-based access control.
               - trafficControl: indicates throttling.
               - backendSignature: indicates backend signature.
               - jwtAuth: indicates JWT (OpenId Connect).
               - cors: indicates cross-origin resource access (CORS).
               - caching: indicates caching.
        :param pulumi.Input[Mapping[str, Any]] tags: A mapping of tags to assign to the resource.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if plugin_data is not None:
            pulumi.set(__self__, "plugin_data", plugin_data)
        if plugin_name is not None:
            pulumi.set(__self__, "plugin_name", plugin_name)
        if plugin_type is not None:
            pulumi.set(__self__, "plugin_type", plugin_type)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the plug-in, which cannot exceed 200 characters.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="pluginData")
    def plugin_data(self) -> Optional[pulumi.Input[str]]:
        """
        The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        """
        return pulumi.get(self, "plugin_data")

    @plugin_data.setter
    def plugin_data(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "plugin_data", value)

    @property
    @pulumi.getter(name="pluginName")
    def plugin_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        """
        return pulumi.get(self, "plugin_name")

    @plugin_name.setter
    def plugin_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "plugin_name", value)

    @property
    @pulumi.getter(name="pluginType")
    def plugin_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
        - ipControl: indicates IP address-based access control.
        - trafficControl: indicates throttling.
        - backendSignature: indicates backend signature.
        - jwtAuth: indicates JWT (OpenId Connect).
        - cors: indicates cross-origin resource access (CORS).
        - caching: indicates caching.
        """
        return pulumi.get(self, "plugin_type")

    @plugin_type.setter
    def plugin_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "plugin_type", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "tags", value)


class Plugin(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 plugin_data: Optional[pulumi.Input[str]] = None,
                 plugin_name: Optional[pulumi.Input[str]] = None,
                 plugin_type: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 __props__=None):
        """
        Provides a Api Gateway Plugin resource.

        For information about Api Gateway Plugin and how to use it, see [What is Plugin](https://www.alibabacloud.com/help/en/api-gateway/latest/create-an-plugin).

        > **NOTE:** Available since v1.187.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        default = alicloud.apigateway.Plugin("default",
            description="tf_example",
            plugin_data="{\\"allowOrigins\\": \\"api.foo.com\\",\\"allowMethods\\": \\"GET,POST,PUT,DELETE,HEAD,OPTIONS,PATCH\\",\\"allowHeaders\\": \\"Authorization,Accept,Accept-Ranges,Cache-Control,Range,Date,Content-Type,Content-Length,Content-MD5,User-Agent,X-Ca-Signature,X-Ca-Signature-Headers,X-Ca-Signature-Method,X-Ca-Key,X-Ca-Timestamp,X-Ca-Nonce,X-Ca-Stage,X-Ca-Request-Mode,x-ca-deviceid\\",\\"exposeHeaders\\": \\"Content-MD5,Server,Date,Latency,X-Ca-Request-Id,X-Ca-Error-Code,X-Ca-Error-Message\\",\\"maxAge\\": 172800,\\"allowCredentials\\": true}",
            plugin_name="tf_example",
            plugin_type="cors",
            tags={
                "Created": "TF",
                "For": "example",
            })
        ```

        ## Import

        Api Gateway Plugin can be imported using the id, e.g.

        ```sh
         $ pulumi import alicloud:apigateway/plugin:Plugin example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the plug-in, which cannot exceed 200 characters.
        :param pulumi.Input[str] plugin_data: The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        :param pulumi.Input[str] plugin_name: The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        :param pulumi.Input[str] plugin_type: The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
               - ipControl: indicates IP address-based access control.
               - trafficControl: indicates throttling.
               - backendSignature: indicates backend signature.
               - jwtAuth: indicates JWT (OpenId Connect).
               - cors: indicates cross-origin resource access (CORS).
               - caching: indicates caching.
        :param pulumi.Input[Mapping[str, Any]] tags: A mapping of tags to assign to the resource.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PluginArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Api Gateway Plugin resource.

        For information about Api Gateway Plugin and how to use it, see [What is Plugin](https://www.alibabacloud.com/help/en/api-gateway/latest/create-an-plugin).

        > **NOTE:** Available since v1.187.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        default = alicloud.apigateway.Plugin("default",
            description="tf_example",
            plugin_data="{\\"allowOrigins\\": \\"api.foo.com\\",\\"allowMethods\\": \\"GET,POST,PUT,DELETE,HEAD,OPTIONS,PATCH\\",\\"allowHeaders\\": \\"Authorization,Accept,Accept-Ranges,Cache-Control,Range,Date,Content-Type,Content-Length,Content-MD5,User-Agent,X-Ca-Signature,X-Ca-Signature-Headers,X-Ca-Signature-Method,X-Ca-Key,X-Ca-Timestamp,X-Ca-Nonce,X-Ca-Stage,X-Ca-Request-Mode,x-ca-deviceid\\",\\"exposeHeaders\\": \\"Content-MD5,Server,Date,Latency,X-Ca-Request-Id,X-Ca-Error-Code,X-Ca-Error-Message\\",\\"maxAge\\": 172800,\\"allowCredentials\\": true}",
            plugin_name="tf_example",
            plugin_type="cors",
            tags={
                "Created": "TF",
                "For": "example",
            })
        ```

        ## Import

        Api Gateway Plugin can be imported using the id, e.g.

        ```sh
         $ pulumi import alicloud:apigateway/plugin:Plugin example <id>
        ```

        :param str resource_name: The name of the resource.
        :param PluginArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PluginArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 plugin_data: Optional[pulumi.Input[str]] = None,
                 plugin_name: Optional[pulumi.Input[str]] = None,
                 plugin_type: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PluginArgs.__new__(PluginArgs)

            __props__.__dict__["description"] = description
            if plugin_data is None and not opts.urn:
                raise TypeError("Missing required property 'plugin_data'")
            __props__.__dict__["plugin_data"] = plugin_data
            if plugin_name is None and not opts.urn:
                raise TypeError("Missing required property 'plugin_name'")
            __props__.__dict__["plugin_name"] = plugin_name
            if plugin_type is None and not opts.urn:
                raise TypeError("Missing required property 'plugin_type'")
            __props__.__dict__["plugin_type"] = plugin_type
            __props__.__dict__["tags"] = tags
        super(Plugin, __self__).__init__(
            'alicloud:apigateway/plugin:Plugin',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            plugin_data: Optional[pulumi.Input[str]] = None,
            plugin_name: Optional[pulumi.Input[str]] = None,
            plugin_type: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Mapping[str, Any]]] = None) -> 'Plugin':
        """
        Get an existing Plugin resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the plug-in, which cannot exceed 200 characters.
        :param pulumi.Input[str] plugin_data: The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        :param pulumi.Input[str] plugin_name: The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        :param pulumi.Input[str] plugin_type: The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
               - ipControl: indicates IP address-based access control.
               - trafficControl: indicates throttling.
               - backendSignature: indicates backend signature.
               - jwtAuth: indicates JWT (OpenId Connect).
               - cors: indicates cross-origin resource access (CORS).
               - caching: indicates caching.
        :param pulumi.Input[Mapping[str, Any]] tags: A mapping of tags to assign to the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PluginState.__new__(_PluginState)

        __props__.__dict__["description"] = description
        __props__.__dict__["plugin_data"] = plugin_data
        __props__.__dict__["plugin_name"] = plugin_name
        __props__.__dict__["plugin_type"] = plugin_type
        __props__.__dict__["tags"] = tags
        return Plugin(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        The description of the plug-in, which cannot exceed 200 characters.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="pluginData")
    def plugin_data(self) -> pulumi.Output[str]:
        """
        The definition statement of the plug-in. Plug-in definition statements in the JSON and YAML formats are supported.
        """
        return pulumi.get(self, "plugin_data")

    @property
    @pulumi.getter(name="pluginName")
    def plugin_name(self) -> pulumi.Output[str]:
        """
        The name of the plug-in that you want to create. It can contain uppercase English letters, lowercase English letters, Chinese characters, numbers, and underscores (_). It must be 4 to 50 characters in length and cannot start with an underscore (_).
        """
        return pulumi.get(self, "plugin_name")

    @property
    @pulumi.getter(name="pluginType")
    def plugin_type(self) -> pulumi.Output[str]:
        """
        The type of the plug-in. Valid values: `backendSignature`, `caching`, `cors`, `ipControl`, `jwtAuth`, `trafficControl`.
        - ipControl: indicates IP address-based access control.
        - trafficControl: indicates throttling.
        - backendSignature: indicates backend signature.
        - jwtAuth: indicates JWT (OpenId Connect).
        - cors: indicates cross-origin resource access (CORS).
        - caching: indicates caching.
        """
        return pulumi.get(self, "plugin_type")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Mapping[str, Any]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetVpcEndpointLinkedVpcsResult',
    'AwaitableGetVpcEndpointLinkedVpcsResult',
    'get_vpc_endpoint_linked_vpcs',
    'get_vpc_endpoint_linked_vpcs_output',
]

@pulumi.output_type
class GetVpcEndpointLinkedVpcsResult:
    """
    A collection of values returned by getVpcEndpointLinkedVpcs.
    """
    def __init__(__self__, id=None, ids=None, instance_id=None, module_name=None, output_file=None, status=None, vpc_endpoint_linked_vpcs=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ids and not isinstance(ids, list):
            raise TypeError("Expected argument 'ids' to be a list")
        pulumi.set(__self__, "ids", ids)
        if instance_id and not isinstance(instance_id, str):
            raise TypeError("Expected argument 'instance_id' to be a str")
        pulumi.set(__self__, "instance_id", instance_id)
        if module_name and not isinstance(module_name, str):
            raise TypeError("Expected argument 'module_name' to be a str")
        pulumi.set(__self__, "module_name", module_name)
        if output_file and not isinstance(output_file, str):
            raise TypeError("Expected argument 'output_file' to be a str")
        pulumi.set(__self__, "output_file", output_file)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if vpc_endpoint_linked_vpcs and not isinstance(vpc_endpoint_linked_vpcs, list):
            raise TypeError("Expected argument 'vpc_endpoint_linked_vpcs' to be a list")
        pulumi.set(__self__, "vpc_endpoint_linked_vpcs", vpc_endpoint_linked_vpcs)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def ids(self) -> Sequence[str]:
        return pulumi.get(self, "ids")

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> str:
        """
        The ID of the instance.
        """
        return pulumi.get(self, "instance_id")

    @property
    @pulumi.getter(name="moduleName")
    def module_name(self) -> str:
        """
        The name of the module that you want to access.
        """
        return pulumi.get(self, "module_name")

    @property
    @pulumi.getter(name="outputFile")
    def output_file(self) -> Optional[str]:
        return pulumi.get(self, "output_file")

    @property
    @pulumi.getter
    def status(self) -> Optional[str]:
        """
        The status of the Vpc Endpoint Linked Vpc.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="vpcEndpointLinkedVpcs")
    def vpc_endpoint_linked_vpcs(self) -> Sequence['outputs.GetVpcEndpointLinkedVpcsVpcEndpointLinkedVpcResult']:
        """
        A list of CR Vpc Endpoint Linked Vpcs. Each element contains the following attributes:
        """
        return pulumi.get(self, "vpc_endpoint_linked_vpcs")


class AwaitableGetVpcEndpointLinkedVpcsResult(GetVpcEndpointLinkedVpcsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetVpcEndpointLinkedVpcsResult(
            id=self.id,
            ids=self.ids,
            instance_id=self.instance_id,
            module_name=self.module_name,
            output_file=self.output_file,
            status=self.status,
            vpc_endpoint_linked_vpcs=self.vpc_endpoint_linked_vpcs)


def get_vpc_endpoint_linked_vpcs(ids: Optional[Sequence[str]] = None,
                                 instance_id: Optional[str] = None,
                                 module_name: Optional[str] = None,
                                 output_file: Optional[str] = None,
                                 status: Optional[str] = None,
                                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetVpcEndpointLinkedVpcsResult:
    """
    This data source provides the CR Vpc Endpoint Linked Vpcs of the current Alibaba Cloud user.

    > **NOTE:** Available in v1.199.0+.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    ids = alicloud.cr.get_vpc_endpoint_linked_vpcs(ids=["example_id"],
        instance_id="your_cr_instance_id",
        module_name="Registry")
    pulumi.export("alicloudCrVpcEndpointLinkedVpcsId1", ids.vpc_endpoint_linked_vpcs[0].id)
    ```


    :param Sequence[str] ids: A list of CR Vpc Endpoint Linked Vpc IDs.
    :param str instance_id: The ID of the instance.
    :param str module_name: The name of the module that you want to access. Valid Values:
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str status: The status of the Vpc Endpoint Linked Vpc. Valid Values: `CREATING`, `RUNNING`.
    """
    __args__ = dict()
    __args__['ids'] = ids
    __args__['instanceId'] = instance_id
    __args__['moduleName'] = module_name
    __args__['outputFile'] = output_file
    __args__['status'] = status
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('alicloud:cr/getVpcEndpointLinkedVpcs:getVpcEndpointLinkedVpcs', __args__, opts=opts, typ=GetVpcEndpointLinkedVpcsResult).value

    return AwaitableGetVpcEndpointLinkedVpcsResult(
        id=pulumi.get(__ret__, 'id'),
        ids=pulumi.get(__ret__, 'ids'),
        instance_id=pulumi.get(__ret__, 'instance_id'),
        module_name=pulumi.get(__ret__, 'module_name'),
        output_file=pulumi.get(__ret__, 'output_file'),
        status=pulumi.get(__ret__, 'status'),
        vpc_endpoint_linked_vpcs=pulumi.get(__ret__, 'vpc_endpoint_linked_vpcs'))


@_utilities.lift_output_func(get_vpc_endpoint_linked_vpcs)
def get_vpc_endpoint_linked_vpcs_output(ids: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                        instance_id: Optional[pulumi.Input[str]] = None,
                                        module_name: Optional[pulumi.Input[str]] = None,
                                        output_file: Optional[pulumi.Input[Optional[str]]] = None,
                                        status: Optional[pulumi.Input[Optional[str]]] = None,
                                        opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetVpcEndpointLinkedVpcsResult]:
    """
    This data source provides the CR Vpc Endpoint Linked Vpcs of the current Alibaba Cloud user.

    > **NOTE:** Available in v1.199.0+.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    ids = alicloud.cr.get_vpc_endpoint_linked_vpcs(ids=["example_id"],
        instance_id="your_cr_instance_id",
        module_name="Registry")
    pulumi.export("alicloudCrVpcEndpointLinkedVpcsId1", ids.vpc_endpoint_linked_vpcs[0].id)
    ```


    :param Sequence[str] ids: A list of CR Vpc Endpoint Linked Vpc IDs.
    :param str instance_id: The ID of the instance.
    :param str module_name: The name of the module that you want to access. Valid Values:
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str status: The status of the Vpc Endpoint Linked Vpc. Valid Values: `CREATING`, `RUNNING`.
    """
    ...
